import 'dart:async';

abstract class DataStore<ModelType, IdType> {
  /// Returns a stream of all records
  Stream<ModelType> getAll();

  /// Returns a single record by id [id]
  Future<ModelType> getById(IdType id);

  /// Inserts a new record [object]
  Future<IdType> insert(ModelType object);

  /// Updates the record with id [id] with the data [object]
  Future<ModelType> updateById(String id, ModelType object);

  /// Deletes a record by id [id]
  Future<Null> deleteById(String id);
}

abstract class DataStoreOffsetPaginated<ModelType, IdType>
    implements DataStore<ModelType, IdType> {
  /// Returns a stream of maximum [count] number of records after the offset [offset]
  Stream<ModelType> getPaginated(int offset, int count);
}

abstract class DataStoreIdPaginated<ModelType, IdType>
    implements DataStore<ModelType, IdType> {
  /// Returns a Stream of maximum [count] number of records after the id [after]
  Stream<ModelType> getPaginated(IdType after, int count);
}